@@ -175,6 +175,7 @@ void canvas_image_move(canvas_item_t *item, gint x, gint y,
 	       NULL);  
 }
 
+/* get the polygon/polyway segment a certain coordinate is over */
 gint canvas_item_get_segment(canvas_item_t *item, gint x, gint y) {
 
   canvas_points_t *points = NULL;
@@ -226,3 +227,19 @@ gint canvas_item_get_segment(canvas_item_t *item, gint x, gint y) {
 
   return retval;
 }
+
+void canvas_item_get_segment_pos(canvas_item_t *item, gint seg,
+				 gint *x0, gint *y0, gint *x1, gint *y1) {
+  printf("get segment %d of item %p\n", seg, item);
+
+  canvas_points_t *points = NULL;
+  g_object_get(G_OBJECT(item), "points", &points, NULL);
+
+  g_assert(points);
+  g_assert(seg < points->num_points-1);
+
+  *x0 = points->coords[2 * seg + 0];
+  *y0 = points->coords[2 * seg + 1];
+  *x1 = points->coords[2 * seg + 2];
+  *y1 = points->coords[2 * seg + 3];
+}
