@@ -114,12 +114,6 @@ static void on_tag_remove(GtkWidget *but, tag_context_t *context) {
     gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 
     update_collisions(context->store, *context->tag);
-
-    /* but is NULL if remove has been called due to a cancelled add */
-    if(but) {
-      /* something has been removed -> now dirty */
-      map_item_set_flags(&context->appdata->map->selected, OSM_FLAG_DIRTY, 0);
-    }
   }
   
   /* disable remove and edit buttons */
@@ -205,8 +199,7 @@ static gboolean tag_edit(tag_context_t *context) {
 }
 
 static void on_tag_edit(GtkWidget *button, tag_context_t *context) {
-  if(tag_edit(context))
-    map_item_set_flags(&context->appdata->map->selected, OSM_FLAG_DIRTY, 0);
+  tag_edit(context);
 }
 
 static void on_tag_last(GtkWidget *button, tag_context_t *context) {
@@ -228,7 +221,6 @@ static void on_tag_last(GtkWidget *button, tag_context_t *context) {
       *context->tag = osm_tags_copy(context->appdata->map->last_way_tags, TRUE);
 
     info_tags_replace(context);
-    map_item_set_flags(&context->appdata->map->selected, OSM_FLAG_DIRTY, 0);
   }
 }
 
@@ -265,14 +257,10 @@ static void on_tag_add(GtkWidget *button, tag_context_t *context) {
   if(!tag_edit(context)) {
     printf("cancelled\n");
     on_tag_remove(NULL, context);
-  } else
-    map_item_set_flags(&context->appdata->map->selected, 
-		       OSM_FLAG_DIRTY, 0);
+  }
 }
 
 void info_tags_replace(tag_context_t *context) {
-  map_item_set_flags(&context->appdata->map->selected, OSM_FLAG_DIRTY, 0);
-
   gtk_list_store_clear(context->store);
 
   GtkTreeIter iter;
@@ -401,8 +389,12 @@ static GtkWidget *tag_widget(tag_context_t *context) {
   return vbox;
 }
 
-void info_dialog(appdata_t *appdata) {
-  g_assert(appdata->map->selected.type != MAP_TYPE_ILLEGAL);
+/* edit tags of currently selected node or way or of the relation */
+/* given */
+void info_dialog(GtkWidget *parent, appdata_t *appdata, relation_t *relation) {
+  if(!relation)
+    g_assert(appdata->map->selected.type != MAP_TYPE_ILLEGAL);
+
   tag_context_t *context = g_new0(tag_context_t, 1);
   user_t *user = NULL;
   char *str = NULL;
@@ -412,29 +404,37 @@ void info_dialog(appdata_t *appdata) {
   context->appdata = appdata;
   context->tag = &work_copy;
 
-  switch(appdata->map->selected.type) {
-  case MAP_TYPE_NODE:
-    str = g_strdup_printf(_("Node #%ld"), appdata->map->selected.node->id);
-    user = appdata->map->selected.node->user;
-    work_copy = osm_tags_copy(appdata->map->selected.node->tag, FALSE);
-    stime = appdata->map->selected.node->time;
-    context->type = NODE;
-    break;
-  case MAP_TYPE_WAY:
-    str = g_strdup_printf(_("Way #%ld"), appdata->map->selected.way->id);
-    user = appdata->map->selected.way->user;
-    work_copy = osm_tags_copy(appdata->map->selected.way->tag, FALSE);
-    stime = appdata->map->selected.way->time;
-    context->type = WAY;
-    break;
-  default:
-    g_assert((appdata->map->selected.type == MAP_TYPE_NODE) ||
-	     (appdata->map->selected.type == MAP_TYPE_WAY));
-    break;
+  if(!relation) {
+    switch(appdata->map->selected.type) {
+    case MAP_TYPE_NODE:
+      str = g_strdup_printf(_("Node #%ld"), appdata->map->selected.node->id);
+      user = appdata->map->selected.node->user;
+      work_copy = osm_tags_copy(appdata->map->selected.node->tag, FALSE);
+      stime = appdata->map->selected.node->time;
+      context->type = NODE;
+      break;
+    case MAP_TYPE_WAY:
+      str = g_strdup_printf(_("Way #%ld"), appdata->map->selected.way->id);
+      user = appdata->map->selected.way->user;
+      work_copy = osm_tags_copy(appdata->map->selected.way->tag, FALSE);
+      stime = appdata->map->selected.way->time;
+      context->type = WAY;
+      break;
+    default:
+      g_assert((appdata->map->selected.type == MAP_TYPE_NODE) ||
+	       (appdata->map->selected.type == MAP_TYPE_WAY));
+      break;
+    }
+  } else {
+    str = g_strdup_printf(_("Relation #%ld"), relation->id);
+    user = relation->user;
+    work_copy = osm_tags_copy(relation->tag, FALSE);
+    stime = relation->time;
+    context->type = RELATION;
   }
 
   context->dialog = gtk_dialog_new_with_buttons(str,
-	  GTK_WINDOW(appdata->window), GTK_DIALOG_MODAL,
+	  GTK_WINDOW(parent), GTK_DIALOG_MODAL,
 	  GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, 
 	  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
 	  NULL);
@@ -472,27 +472,60 @@ void info_dialog(appdata_t *appdata) {
   gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 0, 1);
 
   /* ------------ coordinate (only for nodes) ----------------- */
-  if(appdata->map->selected.type == MAP_TYPE_NODE) {
-    char pos_str[32];
-    pos_lat_str(pos_str, sizeof(pos_str),appdata->map->selected.node->pos.lat);
-    label = gtk_label_new(pos_str);
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
-    pos_lat_str(pos_str, sizeof(pos_str),appdata->map->selected.node->pos.lon);
-    label = gtk_label_new(pos_str);
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
-  } else {
-    char *nodes_str = g_strdup_printf(_("Length: %u nodes"), 
+  if(!relation) {
+    if(appdata->map->selected.type == MAP_TYPE_NODE) {
+      char pos_str[32];
+      pos_lat_str(pos_str, sizeof(pos_str),appdata->map->selected.node->pos.lat);
+      label = gtk_label_new(pos_str);
+      gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
+      pos_lat_str(pos_str, sizeof(pos_str),appdata->map->selected.node->pos.lon);
+      label = gtk_label_new(pos_str);
+      gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
+    } else {
+      char *nodes_str = g_strdup_printf(_("Length: %u nodes"), 
 		osm_way_number_of_nodes(appdata->map->selected.way));
-    label = gtk_label_new(nodes_str);
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
-    g_free(nodes_str);
+      label = gtk_label_new(nodes_str);
+      gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
+      g_free(nodes_str);
+      
+      if(appdata->map->selected.way->draw_flags & OSM_DRAW_FLAG_POLYGON) 
+	label = gtk_label_new("Closed way (polygon)");
+      else
+	label = gtk_label_new("Open way (line)");
+      
+      gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
+    }
+  } else {
+    /* relations tell something about their members */
+    gint nodes = 0, ways = 0, relations = 0;
+    member_t *member = relation->member;
+    while(member) {
+      switch(member->type) {
+      case NODE:
+      case NODE_ID:
+	nodes++;
+	break;
+      case WAY:
+      case WAY_ID:
+	ways++;
+	break;
+      case RELATION:
+      case RELATION_ID:
+	relations++;
+	break;
 
-    if(appdata->map->selected.way->draw_flags & OSM_DRAW_FLAG_POLYGON) 
-      label = gtk_label_new("Closed way (polygon)");
-    else
-      label = gtk_label_new("Open way (line)");
+      default:
+	break;
+      }
 
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
+      member = member->next;
+    }
+
+    char *str = g_strdup_printf(_("Members: %d nodes, %d ways, %d relations"),
+				nodes, ways, relations);
+
+    gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(str), 0, 2, 1, 2);
+    g_free(str);
   }
 
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(context->dialog)->vbox), table, 
@@ -510,27 +543,33 @@ void info_dialog(appdata_t *appdata) {
   if(gtk_dialog_run(GTK_DIALOG(context->dialog)) == GTK_RESPONSE_ACCEPT) {
     gtk_widget_destroy(context->dialog);
 
-    /* replace original tags with work copy */
-    switch(appdata->map->selected.type) {
+    if(!relation) {
+      /* replace original tags with work copy */
+      switch(appdata->map->selected.type) {
 
-    case MAP_TYPE_NODE:
-      osm_tags_free(appdata->map->selected.node->tag);
-      appdata->map->selected.node->tag = osm_tags_copy(work_copy, TRUE);
-      break;
+      case MAP_TYPE_NODE:
+	osm_tags_free(appdata->map->selected.node->tag);
+	appdata->map->selected.node->tag = osm_tags_copy(work_copy, TRUE);
+	break;
+	
+      case MAP_TYPE_WAY:
+	osm_tags_free(appdata->map->selected.way->tag);
+	appdata->map->selected.way->tag = osm_tags_copy(work_copy, TRUE);
+	break;
+	
+      default:
+	break;
+      }
 
-    case MAP_TYPE_WAY:
-      osm_tags_free(appdata->map->selected.way->tag);
-      appdata->map->selected.way->tag = osm_tags_copy(work_copy, TRUE);
-      break;
-
-    default:
-      break;
+      /* since nodes being parts of ways but with no tags are invisible, */
+      /* the result of editing them may have changed their visibility */
+      map_item_redraw(appdata, &appdata->map->selected);
+      map_item_set_flags(&context->appdata->map->selected, OSM_FLAG_DIRTY, 0);
+    } else {
+      osm_tags_free(relation->tag);
+      relation->tag = osm_tags_copy(work_copy, TRUE);
+      relation->flags |= OSM_FLAG_DIRTY;
     }
-
-    /* since nodes being parts of ways but with no tags are invisible, */
-    /* the result of editing them may have changed their visibility */
-    map_item_redraw(appdata, &appdata->map->selected);
-
   } else {
     gtk_widget_destroy(context->dialog);
     osm_tags_free(work_copy);
