@@ -539,6 +539,69 @@ void map_item_redraw(appdata_t *appdata, map_item_t *map_item) {
     map_item_select(appdata, &item);
 }
 
+/* Draw the frisket area which masks off areas it'd be unsafe to edit,
+ * plus its inner edge marker line */
+void map_frisket_draw(map_t *map, bounds_t *bounds) {
+
+    // Edge marker line
+    gint ew2 = FRISKET_EDGE_WIDTH/2;
+    lpos_t i2a; i2a.x = bounds->min.x-ew2; i2a.y = bounds->min.y-ew2;
+    lpos_t i2b; i2b.x = bounds->max.x+ew2; i2b.y = bounds->min.y-ew2;
+    lpos_t i2c; i2c.x = bounds->max.x+ew2; i2c.y = bounds->max.y+ew2;
+    lpos_t i2d; i2d.x = bounds->min.x-ew2; i2d.y = bounds->max.y+ew2;
+
+    // Box inner window
+    lpos_t ia; ia.x = bounds->min.x; ia.y = bounds->min.y;
+    lpos_t ib; ib.x = bounds->max.x; ib.y = bounds->min.y;
+    lpos_t ic; ic.x = bounds->max.x; ic.y = bounds->max.y;
+    lpos_t id; id.x = bounds->min.x; id.y = bounds->max.y;
+
+    // Box outer edge
+    lpos_t oa; oa.x = FRISKET_MULT*ia.x; oa.y = FRISKET_MULT*ia.y;
+    lpos_t ob; ob.x = FRISKET_MULT*ib.x; ob.y = FRISKET_MULT*ib.y;
+    lpos_t oc; oc.x = FRISKET_MULT*ic.x; oc.y = FRISKET_MULT*ic.y;
+    lpos_t od; od.x = FRISKET_MULT*id.x; od.y = FRISKET_MULT*id.y;
+
+    canvas_points_t *points;
+
+    // Outer frisket matte
+    points = canvas_points_new(7);
+    canvas_point_set_pos(points, 0, &oa);
+    canvas_point_set_pos(points, 1, &ob);
+    canvas_point_set_pos(points, 2, &oc);
+    canvas_point_set_pos(points, 3, &ic);
+    canvas_point_set_pos(points, 4, &ib);
+    canvas_point_set_pos(points, 5, &ia);
+    canvas_point_set_pos(points, 6, &oa);
+    canvas_polygon_new(map, CANVAS_GROUP_NODES, points,
+                       1, NO_COLOR, FRISKET_COLOR);
+    canvas_points_free(points);
+
+    points = canvas_points_new(7);
+    canvas_point_set_pos(points, 0, &oc);
+    canvas_point_set_pos(points, 1, &od);
+    canvas_point_set_pos(points, 2, &oa);
+    canvas_point_set_pos(points, 3, &ia);
+    canvas_point_set_pos(points, 4, &id);
+    canvas_point_set_pos(points, 5, &ic);
+    canvas_point_set_pos(points, 6, &oc);
+    canvas_polygon_new(map, CANVAS_GROUP_NODES, points,
+                       1, NO_COLOR, FRISKET_COLOR);
+    canvas_points_free(points);
+    
+    // Frisket inner edge - pick out in a slightly darker colour
+    points = canvas_points_new(5);
+    canvas_point_set_pos(points, 0, &i2a);
+    canvas_point_set_pos(points, 1, &i2b);
+    canvas_point_set_pos(points, 2, &i2c);
+    canvas_point_set_pos(points, 3, &i2d);
+    canvas_point_set_pos(points, 4, &i2a);
+    canvas_polyline_new(map, CANVAS_GROUP_NODES, points,
+                        FRISKET_EDGE_WIDTH, FRISKET_EDGE_COLOR);
+
+    canvas_points_free(points);
+}
+
 static void map_draw(map_t *map, osm_t *osm) {
   g_assert(map->canvas);
 
@@ -555,6 +618,9 @@ static void map_draw(map_t *map, osm_t *osm) {
     map_node_draw(map, node);
     node = node->next;
   }
+
+  printf("drawing frisket...\n");
+  map_frisket_draw(map, osm->bounds);
 }
 
 void map_state_free(map_state_t *state) {
@@ -701,17 +767,180 @@ static map_item_t *map_real_item_at(map_t *map, gint x, gint y) {
   return map_item;
 }
 
+
+
+#ifdef USE_GOOCANVAS
+
+/* Limitations on the amount by which we can scroll. Keeps part of the
+ * map visible at all times */
+static void map_limit_scroll(map_t *map, gint *sx, gint *sy) {
+    gdouble zoom = goo_canvas_get_scale(GOO_CANVAS(map->canvas));
+
+    gint sx_cu = *sx / zoom;
+    gint sy_cu = *sy / zoom;
+
+    // Canvas viewport dimensions
+    GtkAllocation *a = &GTK_WIDGET(map->canvas)->allocation;
+    gint aw_cu = a->width / zoom;
+    gint ah_cu = a->height / zoom;
+
+    // Data rect minimum and maximum
+    gint min_x, min_y, max_x, max_y;
+    min_x = map->appdata->osm->bounds->min.x;
+    min_y = map->appdata->osm->bounds->min.y;
+    max_x = map->appdata->osm->bounds->max.x;
+    max_y = map->appdata->osm->bounds->max.y;
+
+    // limit stops - prevent scrolling beyond these
+    gint min_sy_cu = 0.95*(min_y - ah_cu);
+    gint min_sx_cu = 0.95*(min_x - aw_cu);
+    gint max_sy_cu = 0.95*(max_y);
+    gint max_sx_cu = 0.95*(max_x);
+    if (sy_cu < min_sy_cu) { *sy = min_sy_cu*zoom; }
+    if (sx_cu < min_sx_cu) { *sx = min_sx_cu*zoom; }
+    if (sy_cu > max_sy_cu) { *sy = max_sy_cu*zoom; }
+    if (sx_cu > max_sx_cu) { *sx = max_sx_cu*zoom; }
+}
+
+
+/* Limit a proposed zoom factor to sane ranges.
+ * Specifically the map is allowed to be no smaller than the viewport. */
+static gboolean map_limit_zoom(map_t *map, gdouble *zoom) {
+    // Data rect minimum and maximum
+    gint min_x, min_y, max_x, max_y;
+    min_x = map->appdata->osm->bounds->min.x;
+    min_y = map->appdata->osm->bounds->min.y;
+    max_x = map->appdata->osm->bounds->max.x;
+    max_y = map->appdata->osm->bounds->max.y;
+
+    // Canvas viewport dimensions
+    GtkAllocation *a = &GTK_WIDGET(map->canvas)->allocation;
+    gint ah_cu = a->height / *zoom;
+    gint aw_cu = a->width / *zoom;
+
+    gdouble oldzoom = *zoom;
+    if (ah_cu < aw_cu) {
+        gint lim_h = ah_cu*0.95;
+        if (max_y-min_y < lim_h) {
+            gdouble corr = ((gdouble)max_y-min_y) / (gdouble)lim_h;
+            *zoom /= corr;
+        }
+    }
+    else {
+        gint lim_w = aw_cu*0.95;
+        if (max_x-min_x < lim_w) {
+            gdouble corr = ((gdouble)max_x-min_x) / (gdouble)lim_w;
+            *zoom /= corr;
+        }
+    }
+    if (*zoom != oldzoom) {
+        printf("Can't zoom further out\n");
+        return 1;
+    }
+    return 0;
+}
+
+
+/* Scroll the map a little towards the centre from where it is right now.
+ * This is used as a cute recentring trick when the map is at its outer
+ * scroll limit. */
+static void map_scroll_towards_centre(map_t *map, gdouble amt) {
+    gint sx, sy, sx_orig, sy_orig;
+    canvas_get_scroll_offsets(map->canvas, &sx, &sy);
+    gdouble zoom = goo_canvas_get_scale(GOO_CANVAS(map->canvas));
+    sx_orig=sx;
+    sy_orig=sy;
+
+    // Work in canvas units
+    gdouble sx_cu = sx / zoom;
+    gdouble sy_cu = sy / zoom;
+
+    // Map bounds
+    gdouble bmin_x_cu, bmin_y_cu, bmax_x_cu, bmax_y_cu;
+    bmin_x_cu = map->appdata->osm->bounds->min.x;
+    bmin_y_cu = map->appdata->osm->bounds->min.y;
+    bmax_x_cu = map->appdata->osm->bounds->max.x;
+    bmax_y_cu = map->appdata->osm->bounds->max.y;
+
+    // Canvas viewport dimensions
+    GtkAllocation *a = &GTK_WIDGET(map->canvas)->allocation;
+    gdouble ah_cu = a->height / zoom;
+    gdouble aw_cu = a->width / zoom;
+
+    // Scroll offsets that would recentre the map
+    gdouble centre_sx_cu, centre_sy_cu;
+    centre_sx_cu = - (aw_cu/2);
+    centre_sy_cu = - (ah_cu/2);
+
+    // Move towards centre by a given fraction of the whole map
+    if (sx_cu > centre_sx_cu) {
+        sx_cu -= ((bmax_x_cu - bmin_x_cu) * amt);
+        if (sx_cu < centre_sx_cu) {
+            printf("force-centre-x\n");
+            sx_cu = centre_sx_cu;
+        }
+    }
+    if (sx_cu < centre_sx_cu) {
+        sx_cu += ((bmax_x_cu - bmin_x_cu) * amt);
+        if (sx_cu > centre_sx_cu) {
+            printf("force-centre-x\n");
+            sx_cu = centre_sx_cu;
+        }
+    }
+
+    if (sy_cu > centre_sy_cu) {
+        sy_cu -= ((bmax_y_cu - bmin_y_cu) * amt);
+        if (sy_cu < centre_sy_cu) {
+            printf("force-centre-y\n");
+            sy_cu = centre_sy_cu;
+        }
+    }
+    if (sy_cu < centre_sy_cu) {
+        sy_cu += ((bmax_y_cu - bmin_y_cu) * amt);
+        if (sy_cu > centre_sy_cu) {
+            printf("force-centre-y\n");
+            sy_cu = centre_sy_cu;
+        }
+    }
+
+    // Back to pixels for setting the scroll
+    sx = (gint)(sx_cu * zoom);
+    sy = (gint)(sy_cu * zoom);
+    canvas_scroll_to(map->canvas, sx, sy);
+    map->state->scroll_offset.x = sx;
+    map->state->scroll_offset.y = sy;
+}
+
+#endif
+
+
+
 void map_set_zoom(map_t *map, double zoom, 
 		  gboolean update_scroll_offsets) {
+  gboolean at_zoom_limit = 0;
+#ifdef USE_GOOCANVAS
+  at_zoom_limit = map_limit_zoom(map, &zoom);
+#endif
   map->state->zoom = zoom;
   canvas_set_zoom(map->canvas, map->state->zoom);
 
-  if(update_scroll_offsets) {
-    /* zooming affects the scroll offsets */
-    gint sx, sy;
-    canvas_get_scroll_offsets(map->canvas, &sx, &sy);
-    map->state->scroll_offset.x = sx;
-    map->state->scroll_offset.y = sy;
+  if (update_scroll_offsets) {
+    if (!at_zoom_limit) {
+      /* zooming affects the scroll offsets */
+      gint sx, sy;
+      canvas_get_scroll_offsets(map->canvas, &sx, &sy);
+#ifdef USE_GOOCANVAS
+      map_limit_scroll(map, &sx, &sy);
+      canvas_scroll_to(map->canvas, sx, sy);  // keep the map visible
+#endif
+      map->state->scroll_offset.x = sx;
+      map->state->scroll_offset.y = sy;
+    }
+#ifdef USE_GOOCANVAS
+    else {
+      //      map_scroll_towards_centre(map, 0.20); 
+    }
+#endif
   }
 }
 
@@ -756,18 +985,27 @@ static void map_do_scroll(map_t *map, gint x, gint y) {
   canvas_get_scroll_offsets(map->canvas, &sx, &sy);
   sx -= x-map->pen_down.at.x;
   sy -= y-map->pen_down.at.y;
+#ifdef USE_GOOCANVAS
+  map_limit_scroll(map, &sx, &sy);
+#endif
   canvas_scroll_to(map->canvas, sx, sy);
   map->state->scroll_offset.x = sx;
   map->state->scroll_offset.y = sy;
 }
 
+
 /* scroll a certain step */
 static void map_do_scroll_step(map_t *map, gint x, gint y) {
   gint sx, sy;
   canvas_get_scroll_offsets(map->canvas, &sx, &sy);
-  canvas_scroll_to(map->canvas, sx+x, sy+y);
-  map->state->scroll_offset.x = sx+x;
-  map->state->scroll_offset.y = sy+y;
+  sx += x;
+  sy += y;
+#ifdef USE_GOOCANVAS
+  map_limit_scroll(map, &sx, &sy);
+#endif
+  canvas_scroll_to(map->canvas, sx, sy);
+  map->state->scroll_offset.x = sx;
+  map->state->scroll_offset.y = sy;
 }
 
 
@@ -892,331 +1130,6 @@ void map_way_delete(appdata_t *appdata, way_t *way) {
   osm_way_delete(appdata->osm, way, FALSE);
 }
 
-static void map_node_move(appdata_t *appdata, map_item_t *map_item, 
-			  gint ex, gint ey) {
-
-  map_t *map = appdata->map;
-  osm_t *osm = appdata->osm;
-
-  g_assert(map_item->type == MAP_TYPE_NODE);
-  node_t *node = map_item->node;
-
-  printf("released dragged node #%ld\n", node->id);
-  printf("  was at %d %d (%f %f)\n", 
-	 node->lpos.x, node->lpos.y,
-	 node->pos.lat, node->pos.lon);
-
-
-  /* check if it was dropped onto another node */
-  node_t *touchnode = map_hl_touchnode_get_node(map);
-  if(touchnode) {
-    map_hl_touchnode_clear(map);
-
-    printf("  dropped onto node #%ld\n", touchnode->id);
-
-    if(yes_no_f(GTK_WIDGET(appdata->window), 
-		appdata, MISC_AGAIN_ID_JOIN_NODES, 0,
-		_("Join nodes?"),
-		_("Do you want to join the dragged node with the one "
-		  "you dropped it on?"))) {
-
-      /* the touchnode vanishes and is replaced by the node the */
-      /* user dropped onto it */
-
-      /* TODO: move this into a function in osm.c */
-      way_t *way = appdata->osm->way;
-      while(way) {
-	node_chain_t *chain = way->node_chain;
-	while(chain) {
-	  if(chain->node == touchnode) {
-	    printf("  found node in way #%ld\n", way->id);
-	    
-	    /* replace by node */
-	    chain->node = node;
-	    
-	    /* and adjust way references of both nodes */
-	    node->ways++;
-	    touchnode->ways--;
-	    
-	    way->flags |= OSM_FLAG_DIRTY;
-	  }
-	  chain = chain->next;
-	}
-	way = way->next;
-      }
-      
-      /* replace node in relations */
-      relation_t *relation = appdata->osm->relation;
-      while(relation) {
-	member_t *member = relation->member;
-	while(member) {
-	  if(member->type == NODE && member->node == touchnode) {
-	    printf("  found node in relation #%ld\n", relation->id);
-	    
-	    /* replace by node */
-	    member->node = node;
-	    
-	    relation->flags |= OSM_FLAG_DIRTY;
-	  }
-	  member = member->next;
-	}
-	relation = relation->next;
-      }
-
-      /* transfer tags from touchnode to node */
-      tag_t **dst = &node->tag;
-      while(*dst) dst = &((*dst)->next);  /* find end of target tag list */
-      tag_t **src = &touchnode->tag;
-      gboolean conflict = FALSE;
-      while(*src) {
-	/* check if same key but with different value is present, */
-	/* ignoring the created_by tags */
-	if(!osm_is_creator_tag(*src) && 
-	   osm_tag_key_other_value_present(node->tag, *src))
-	  conflict = TRUE;
-
-	/* don't copy "created_by" and "source" tag or tags that already */
-	/* exist in identical form */
-	if(osm_is_creator_tag(*src) || 
-	   osm_tag_key_and_value_present(node->tag, *src)) {
-	  tag_t *next = (*src)->next;
-	  osm_tag_free(*src);
-	  *src = next;
-	} else {
-	  *dst = *src;
-	  *src = (*src)->next;
-	  dst = &((*dst)->next);
-	  *dst = NULL;
-	}
-      }
-
-      /* touchnode must not have any references to ways anymore */
-      g_assert(!touchnode->ways);
-
-      /* delete touchnode */
-      /* remove it visually from the screen */
-      map_item_chain_destroy(&touchnode->map_item_chain);
-
-      /* and remove it from the data structures */
-      osm_node_remove_from_relation(appdata->osm, touchnode);
-      osm_node_delete(appdata->osm, touchnode, FALSE, TRUE);
-      
-      /* and open dialog to resolve tag collisions if necessary */
-      if(conflict) {
-	messagef(GTK_WIDGET(appdata->window), "Node tag conflict", 
-		 "The resulting node contains some conflicting tags. "
-		 "Please solve these.");
-      }
-    
-      /* check whether this will also join two ways */
-      printf("  checking if node is end of way\n");
-      guint ways2join_cnt = 0;
-      way_t *ways2join[2] = { NULL, NULL };
-      way = appdata->osm->way;
-      while(way) {
-	if(osm_way_ends_with_node(way, node)) {
-	  if(ways2join_cnt < 2) 
-	    ways2join[ways2join_cnt] = way;
-
-	  printf("  way #%ld ends with this node\n", way->id);
-	  ways2join_cnt++;
-	}
-	way = way->next;
-      }
-
-      if(ways2join_cnt > 2) {
-	messagef(GTK_WIDGET(appdata->window), _("Too many ways to join"), 
-		 _("More than two ways now end in this node. Joining more "
-		   "than two ways is not yet implemented, sorry"));
-	
-      } else if(ways2join_cnt == 2) {
-	if(yes_no_f(GTK_WIDGET(appdata->window), 
-		    appdata, MISC_AGAIN_ID_JOIN_WAYS, 0,
-		    _("Join ways?"),
-		    _("Do you want to join the dragged way with the one "
-		      "you dropped it on?"))) {
-
-	  printf("  about to join ways #%ld and #%ld\n",
-		 ways2join[0]->id, ways2join[1]->id);
-
-	  /* way[1] gets destroyed and attached to way[0] */
-	  /* so check if way[1] is selected and exchainge ways then */
-	  /* so that way may stay selected */
-	  if((map->selected.type == MAP_TYPE_WAY) && 
-	     (map->selected.way == ways2join[1])) {
-	    printf("  swapping ways to keep selected one alive\n");
-	    way_t *tmp = ways2join[1];
-	    ways2join[1] = ways2join[0];
-	    ways2join[0] = tmp;
-	  }
-
-	  /* take all nodes from way[1] and append them to way[0] */
-	  /* check if we have to append or prepend to way[0] */
-	  gboolean revert = FALSE;
-	  if(ways2join[0]->node_chain->node == node) {
-	    /* make "prepend" to be "append" by reverting way[0] */
-	    printf("  target prepend -> revert\n");	    
-	    revert = TRUE;
-	    osm_way_revert(ways2join[0]);
-	  } 
-	  
-	  /* verify the common node is last in the target way */
-	  node_chain_t *chain = ways2join[0]->node_chain;
-	  while(chain->next) chain = chain->next;
-	  g_assert(chain->node == node);
-	  g_assert(!chain->next);
-	  
-	  /* common node must be first in the chain to attach */
-	  if(ways2join[1]->node_chain->node != node) {
-	    printf("  source revert\n");	    
-	    osm_way_revert(ways2join[1]);
-	  }
-	  
-	  /* verify the common node is first in the source way */
-	  g_assert(ways2join[1]->node_chain->node == node);
-
-	  /* finally append source chain to target */
-	  g_assert(!chain->next);
-	  chain->next = ways2join[1]->node_chain->next;
-
-	  ways2join[1]->node_chain->next = NULL;
-
-	  /* ---------- transfer tags from way[1] to way[0] ----------- */
-	  tag_t **dst = &ways2join[0]->tag;
-	  while(*dst) dst = &((*dst)->next);  /* find end of target tag list */
-	  tag_t **src = &ways2join[1]->tag;
-	  gboolean conflict = FALSE;
-	  while(*src) {
-	    /* check if same key but with different value is present, */
-	    /* ignoring the created_by tags */
-	    if(!osm_is_creator_tag(*src) && 
-	       osm_tag_key_other_value_present(ways2join[0]->tag, *src))
-	      conflict = TRUE;
-
-	    /* don't copy "created_by" and "source" tag or tags that already */
-	    /* exist in identical form */
-	    if(osm_is_creator_tag(*src) || 
-	       osm_tag_key_and_value_present(ways2join[0]->tag, *src)) {
-	      tag_t *next = (*src)->next;
-	      osm_tag_free(*src);
-	      *src = next;
-	    } else {
-	      *dst = *src;
-	      *src = (*src)->next;
-	      dst = &((*dst)->next);
-	      *dst = NULL;
-	    }
-	  }
-
-	  /* ---- transfer relation membership from way[1] to way[0] ----- */
-	  relation_chain_t *rchain = 
-	    osm_way_to_relation(appdata->osm, ways2join[1]);
-
-	  while(rchain) {
-	    relation_chain_t *next = rchain->next;
-	    printf("way[1] is part of relation #%ld\n", rchain->relation->id);
-
-	    /* make way[0] member of the same relation */
-
-	    /* walk member chain. save role of way[1] if its being found. */
-	    /* end search either at end of chain or if way[0] was foind */
-	    /* as it's already a member of that relation */
-	    member_t **member = &rchain->relation->member;
-	    char *role = NULL;
-	    while(*member && 
-		  !(((*member)->type == WAY) && 
-		    ((*member)->way == ways2join[0]))) {
-
-	      /* save role of way[1] */
-	      if(((*member)->type == WAY) && ((*member)->way == ways2join[0]))
-		role = (*member)->role;
-
-	      member = &(*member)->next;
-	    }
-
-	    if(*member) 
-	      printf("  both ways were members of this relation\n");
-	    else {
-	      printf("  adding way[0] to relation\n");
-	      *member = g_new0(member_t, 1);
-	      (*member)->type = WAY;
-	      (*member)->way = ways2join[0];
-	      if(role) (*member)->role = g_strdup(role);
-	      member = &(*member)->next;
-
-	      rchain->relation->flags |= OSM_FLAG_DIRTY;
-	    }
-
-	    g_free(rchain);
-	    rchain = next;
-	  }
-
-	  
-	  /* and open dialog to resolve tag collisions if necessary */
-	  if(conflict) {
-	    messagef(GTK_WIDGET(appdata->window), "Way tag conflict", 
-		     "The resulting way contains some conflicting tags. "
-		     "Please solve these.");
-	  }
-	  
-	  ways2join[0]->flags |= OSM_FLAG_DIRTY;
-	  map_way_delete(appdata, ways2join[1]);
-	}
-      }
-    }
-  }
-
-  /* finally update dragged nodes position */
-
-  /* convert mouse position to canvas (world) position */
-  gint x, y;
-  canvas_window2world(map->canvas, ex, ey, &x, &y);
-  if(!osm_position_within_bounds(appdata->osm, x, y)) {
-    map_outside_error(appdata);
-    return;
-  }
-  
-  node->lpos.x = x;
-  node->lpos.y = y;
-
-  /* convert mouse position to canvas (world) position */
-  canvas_window2world(map->canvas, ex, ey, &node->lpos.x, &node->lpos.y);
-	    
-  /* convert screen position back to ll */
-  lpos2pos(osm->bounds, &node->lpos, &node->pos);
-
-  printf("  now at %d %d (%f %f)\n", 
-	 node->lpos.x, node->lpos.y, node->pos.lat, node->pos.lon);
-  
-  /* now update the visual representation of the node */
-  
-  map_item_chain_destroy(&node->map_item_chain);
-  map_node_draw(map, node);
-
-  /* visually update ways, node is part of */
-  way_t *way = osm->way;
-  while(way) {
-    if(osm_node_in_way(way, node)) {
-      printf("  node is part of way #%ld, redraw!\n", way->id);
-      
-      /* remove prior version of this way */
-      map_item_chain_destroy(&way->map_item_chain);
-
-      /* draw current way */
-      map_way_color(appdata->potlatch, way);
-      map_way_draw(map, way);
-    }
-    
-    way = way->next;
-  }
-  
-  /* and mark the node as dirty */
-  node->flags |= OSM_FLAG_DIRTY;
-
-  /* update highlight */
-  map_highlight_refresh(appdata);
-}
-			  
 static void map_handle_click(appdata_t *appdata, map_t *map) {
 
   /* problem: on_item may be the highlight itself! So store it! */
@@ -1417,7 +1330,7 @@ static void map_button_release(map_t *map, gint x, gint y) {
 	map_hl_cursor_clear(map);
 	
 	/* now actually move the node */
-	map_node_move(map->appdata, map->pen_down.on_item, x, y);
+	map_edit_node_move(map->appdata, map->pen_down.on_item, x, y);
       }
     }
     break;
@@ -1445,7 +1358,7 @@ static void map_button_release(map_t *map, gint x, gint y) {
       map_node_select(map->appdata, node);
 
       /* let the user specify some tags for the new node */
-      info_dialog(map->appdata);
+      info_dialog(GTK_WIDGET(map->appdata->window), map->appdata, NULL);
     }
     break;
 
@@ -1581,6 +1494,7 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
     return FALSE;
 
   if(event->type == GDK_KEY_PRESS) {
+    gdouble zoom = 0;
     switch(event->keyval) {
 
     case GDK_Left:
@@ -1605,7 +1519,7 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
 	map_action_ok(appdata);
       /* otherwise if info is enabled call that */
       else if(GTK_WIDGET_FLAGS(appdata->iconbar->info) & GTK_SENSITIVE)
-	info_dialog(appdata);
+	info_dialog(GTK_WIDGET(appdata->window), appdata, NULL);
       break;
 
     case GDK_Escape:   // same as HILDON_HARDKEY_ESC
@@ -1619,9 +1533,10 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
 #else
     case '+':
 #endif
-      map_set_zoom(appdata->map, 
-		   appdata->map->state->zoom*ZOOM_FACTOR_BUTTON, TRUE);
-      printf("zoom is now %f\n", appdata->map->state->zoom);
+      zoom = appdata->map->state->zoom;
+      zoom *= ZOOM_FACTOR_BUTTON;
+      map_set_zoom(appdata->map, zoom, TRUE);
+      printf("zoom is now %f\n", zoom);
       return TRUE;
       break;
 
@@ -1630,9 +1545,10 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
 #else
     case '-':
 #endif
-      map_set_zoom(appdata->map, 
-		   appdata->map->state->zoom/ZOOM_FACTOR_BUTTON, TRUE);
-      printf("zoom is now %f\n", appdata->map->state->zoom);
+      zoom = appdata->map->state->zoom;
+      zoom /= ZOOM_FACTOR_BUTTON;
+      map_set_zoom(appdata->map, zoom, TRUE);
+      printf("zoom is now %f\n", zoom);
       return TRUE;
       break;
 
@@ -1716,15 +1632,28 @@ GtkWidget *map_new(appdata_t *appdata) {
 
 void map_init(appdata_t *appdata) {
   map_t *map = appdata->map;
+  gint min_x, min_y, max_x, max_y;
+  min_x = min_y = max_x = max_y = 0;
 
   /* set initial zoom */
   map_set_zoom(map, map->state->zoom, FALSE);
   map_init_colors(appdata->potlatch, appdata->osm);
   map_draw(map, appdata->osm);
 
-  canvas_set_bounds(map->canvas,
-	appdata->osm->bounds->min.x, appdata->osm->bounds->min.y,
-	appdata->osm->bounds->max.x, appdata->osm->bounds->max.y);
+  min_x = appdata->osm->bounds->min.x;
+  min_y = appdata->osm->bounds->min.y;
+  max_x = appdata->osm->bounds->max.x;
+  max_y = appdata->osm->bounds->max.y;
+  min_x *= FRISKET_MULT;
+  min_y *= FRISKET_MULT;
+  max_x *= FRISKET_MULT;
+  max_y *= FRISKET_MULT;
+  
+  canvas_set_bounds(map->canvas, min_x, min_y, max_x, max_y);
+
+  //canvas_set_bounds(map->canvas,
+  //	appdata->osm->bounds->min.x, appdata->osm->bounds->min.y,
+  //	appdata->osm->bounds->max.x, appdata->osm->bounds->max.y);
 
   printf("restore scroll offsets %d/%d\n",
 	 map->state->scroll_offset.x, map->state->scroll_offset.y);
@@ -1813,9 +1742,14 @@ void map_action_set(appdata_t *appdata, map_action_t action) {
 
   case MAP_ACTION_WAY_ADD:
     printf("starting new way\n");
+
+    /* remember if there was a way selected */
+    way_t *way_sel = NULL;
+    if(appdata->map->selected.type == MAP_TYPE_WAY)
+      way_sel = appdata->map->selected.way;
+
     map_item_deselect(appdata);
-    g_assert(!appdata->map->action.way);
-    appdata->map->action.way = osm_way_new();
+    map_edit_way_add_begin(appdata->map, way_sel);
     break;
 
   case MAP_ACTION_NODE_ADD:
@@ -1873,7 +1807,12 @@ void map_action_cancel(appdata_t *appdata) {
 void map_action_ok(appdata_t *appdata) {
   map_t *map = appdata->map;
 
-  switch(map->action.type) {
+  /* reset action now as this erases the statusbar and some */
+  /* of the actions may set it */
+  map_action_t type = map->action.type;
+  map_action_set(appdata, MAP_ACTION_IDLE);
+
+  switch(type) {
   case MAP_ACTION_WAY_ADD:
     map_edit_way_add_ok(map);
     break;
@@ -1888,8 +1827,6 @@ void map_action_ok(appdata_t *appdata) {
   default:
     break;
   }
-
-  map_action_set(appdata, MAP_ACTION_IDLE);
 }
 
 /* called from icon "trash" */
