@@ -20,7 +20,7 @@
 #ifndef POTLATCH_CONF_H
 #define POTLATCH_CONF_H
 
-#define COLOR_NONE 0xffffffff
+#define POTLATCH_COLOR_NONE 0xffffffff
 
 typedef struct potlatch_color {
   char *tag;
