@@ -72,12 +72,13 @@ static gboolean project_read(appdata_t *appdata,
 	
 	while(node != NULL) {
 	  if(node->type == XML_ELEMENT_NODE) {
-	    
+
 	    if(strcasecmp((char*)node->name, "desc") == 0) {
 	      str = (char*)xmlNodeListGetString(doc, node->children, 1);
 	      project->desc = g_strdup(str);
 	      printf("desc = %s\n", project->desc);
 	      xmlFree(str);
+
 	    } else if(strcasecmp((char*)node->name, "server") == 0) {
 	      str = (char*)xmlNodeListGetString(doc, node->children, 1);
 	      project->server = g_strdup(str);
@@ -92,8 +93,24 @@ static gboolean project_read(appdata_t *appdata,
 		project->password = g_strdup(str);
 		xmlFree(str);
 	      } 
-	      
+
+	    } else if(project->map_state && 
+		      strcasecmp((char*)node->name, "map") == 0) {
+	      if((str = (char*)xmlGetProp(node, BAD_CAST "zoom"))) {
+		project->map_state->zoom = g_ascii_strtod(str, NULL);
+		xmlFree(str);
+	      } 
+	      if((str = (char*)xmlGetProp(node, BAD_CAST "scroll-offset-x"))) {
+		project->map_state->scroll_offset.x = strtoul(str, NULL, 10);
+		xmlFree(str);
+	      } 
+	      if((str = (char*)xmlGetProp(node, BAD_CAST "scroll-offset-y"))) {
+		project->map_state->scroll_offset.y = strtoul(str, NULL, 10);
+		xmlFree(str);
+	      } 
+
 	    } else if(strcasecmp((char*)node->name, "wms") == 0) {
+
 	      if((str = (char*)xmlGetProp(node, BAD_CAST "server"))) {
 		project->wms_server = g_strdup(str);
 		xmlFree(str);
@@ -125,6 +142,7 @@ static gboolean project_read(appdata_t *appdata,
 		project->min.lon = g_ascii_strtod(str, NULL);
 		xmlFree(str);
 	      }
+
 	    } else if(strcasecmp((char*)node->name, "max") == 0) {
 	      if((str = (char*)xmlGetProp(node, BAD_CAST "lat"))) {
 		project->max.lat = g_ascii_strtod(str, NULL);
@@ -141,7 +159,7 @@ static gboolean project_read(appdata_t *appdata,
       }
     }
   }
-
+  
   /* ------------ set some default that may be missing ----------- */
   /* ------- e.g. from project files saved by old versions ------- */
   if(!project->wms_server)
@@ -154,6 +172,7 @@ static gboolean project_read(appdata_t *appdata,
 }
 
 gboolean project_save(GtkWidget *parent, project_t *project) {
+  char str[32];
   char *project_file = g_strdup_printf("%s%s.proj", 
 		       project->path, project->name);
 
@@ -188,7 +207,6 @@ gboolean project_save(GtkWidget *parent, project_t *project) {
   xmlNewChild(root_node, NULL, BAD_CAST "osm", BAD_CAST project->osm);
 
   node = xmlNewChild(root_node, NULL, BAD_CAST "min", NULL);
-  char str[32];
   g_ascii_dtostr(str, sizeof(str), project->min.lat);
   xmlNewProp(node, BAD_CAST "lat", BAD_CAST str);
   g_ascii_dtostr(str, sizeof(str), project->min.lon);
@@ -200,6 +218,16 @@ gboolean project_save(GtkWidget *parent, project_t *project) {
   g_ascii_dtostr(str, sizeof(str), project->max.lon);
   xmlNewProp(node, BAD_CAST "lon", BAD_CAST str);
 
+  if(project->map_state) {
+    node = xmlNewChild(root_node, NULL, BAD_CAST "map", BAD_CAST NULL);
+    g_ascii_dtostr(str, sizeof(str), project->map_state->zoom);
+    xmlNewProp(node, BAD_CAST "zoom", BAD_CAST str);
+    snprintf(str, sizeof(str), "%d", project->map_state->scroll_offset.x);
+    xmlNewProp(node, BAD_CAST "scroll-offset-x", BAD_CAST str);
+    snprintf(str, sizeof(str), "%d", project->map_state->scroll_offset.y);
+    xmlNewProp(node, BAD_CAST "scroll-offset-y", BAD_CAST str);
+  }
+
   node = xmlNewChild(root_node, NULL, BAD_CAST "wms", NULL);
   xmlNewProp(node, BAD_CAST "server", BAD_CAST project->wms_server);
   xmlNewProp(node, BAD_CAST "path", BAD_CAST project->wms_path);
@@ -232,6 +260,9 @@ void project_free(project_t *project) {
  
   if(project->path)       g_free(project->path);
   if(project->osm)        g_free(project->osm);
+
+  map_state_free(project->map_state);
+
   g_free(project);
 }
 
@@ -312,11 +343,14 @@ enum {
   PROJECT_NUM_COLS
 };
 
-static void view_selected(select_context_t *context, gboolean selected) {
-  gtk_widget_set_sensitive(context->but_remove, selected);
-  gtk_widget_set_sensitive(context->but_edit, selected);
+static void view_selected(select_context_t *context, project_t *project) {
+  gtk_widget_set_sensitive(context->but_remove, project != NULL);
+  gtk_widget_set_sensitive(context->but_edit, project != NULL);
+
+  /* check if the selected project also has a valid osm file */
   gtk_dialog_set_response_sensitive(GTK_DIALOG(context->dialog), 
-	    GTK_RESPONSE_ACCEPT, selected);
+      GTK_RESPONSE_ACCEPT, 
+      project && g_file_test(project->osm, G_FILE_TEST_IS_REGULAR));
 }
 
 static gboolean
@@ -327,8 +361,11 @@ view_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
   GtkTreeIter iter;
     
   if(gtk_tree_model_get_iter(model, &iter, path)) {
+    project_t *project = NULL;
+    gtk_tree_model_get(model, &iter, PROJECT_COL_DATA, &project, -1);
     g_assert(gtk_tree_path_get_depth(path) == 1);
-    view_selected(context, TRUE);
+
+    view_selected(context, project);
   }
   
   return TRUE; /* allow selection state to change */
@@ -392,6 +429,58 @@ static void callback_modified_name(GtkWidget *widget, gpointer data) {
 				    GTK_RESPONSE_ACCEPT, ok);
 }
 
+
+gboolean project_delete(select_context_t *context, project_t *project) {
+
+  /* remove entire directory from disk */
+  GDir *dir = g_dir_open(project->path, 0, NULL);
+  const char *name = NULL;
+  do {
+    if((name = g_dir_read_name(dir))) {
+      char *fullname = g_strdup_printf("%s/%s", project->path, name);
+      g_remove(fullname);
+      g_free(fullname);
+    }
+  } while(name);
+
+  /* remove the projects directory */
+  g_remove(project->path);
+
+  /* remove from view */
+  GtkTreeIter iter;
+  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(context->view));
+  gboolean deleted = FALSE;
+  if(gtk_tree_model_get_iter_first(model, &iter)) {
+    do {
+      project_t *prj = NULL;
+      gtk_tree_model_get(model, &iter, PROJECT_COL_DATA, &prj, -1);
+      if(prj && (prj == project)) {
+	printf("found %s to remove\n", prj->name);
+	/* and remove from store */
+	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+	deleted = TRUE;
+      }
+    } while(!deleted && gtk_tree_model_iter_next(model, &iter));
+  }
+
+  /* de-chain entry from project list */
+  project_t **project_list = &context->project;
+  while(*project_list) {
+    if(*project_list == project) 
+      *project_list = (*project_list)->next;
+    else
+      project_list = &((*project_list)->next);
+  }
+
+  /* free project structure */
+  project_free(project);
+
+  /* disable edit/remove buttons */
+  view_selected(context, NULL);
+
+  return TRUE;
+}
+
 project_t *project_new(select_context_t *context) {
   printf("creating project with default values\n");
 
@@ -462,7 +551,8 @@ project_t *project_new(select_context_t *context) {
   {
     printf("edit cancelled!!\n");
 
-    project_free(project);
+    project_delete(context, project);
+
     project = NULL;
   }
 
@@ -492,57 +582,6 @@ static void on_project_new(GtkButton *button, gpointer data) {
   }
 }
 
-gboolean project_delete(select_context_t *context, project_t *project) {
-
-  /* remove entire directory from disk */
-  GDir *dir = g_dir_open(project->path, 0, NULL);
-  const char *name = NULL;
-  do {
-    if((name = g_dir_read_name(dir))) {
-      char *fullname = g_strdup_printf("%s/%s", project->path, name);
-      g_remove(fullname);
-      g_free(fullname);
-    }
-  } while(name);
-
-  /* remove the projects directory */
-  g_remove(project->path);
-
-  /* remove from view */
-  GtkTreeIter iter;
-  GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(context->view));
-  gboolean deleted = FALSE;
-  if(gtk_tree_model_get_iter_first(model, &iter)) {
-    do {
-      project_t *prj = NULL;
-      gtk_tree_model_get(model, &iter, PROJECT_COL_DATA, &prj, -1);
-      if(prj && (prj == project)) {
-	printf("found %s to remove\n", prj->name);
-	/* and remove from store */
-	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
-	deleted = TRUE;
-      }
-    } while(!deleted && gtk_tree_model_iter_next(model, &iter));
-  }
-
-  /* de-chain entry from project list */
-  project_t **project_list = &context->project;
-  while(*project_list) {
-    if(*project_list == project) 
-      *project_list = (*project_list)->next;
-    else
-      project_list = &((*project_list)->next);
-  }
-
-  /* free project structure */
-  project_free(project);
-
-  /* disable edit/remove buttons */
-  view_selected(context, FALSE);
-
-  return TRUE;
-}
-
 static void on_project_delete(GtkButton *button, gpointer data) {
   select_context_t *context = (select_context_t*)data;
   project_t *project = project_get_selected(context->view);
@@ -741,10 +780,16 @@ void project_filesize(project_context_t *context) {
   printf("Checking size of %s\n", context->project->osm);
 
   if(!g_file_test(context->project->osm, G_FILE_TEST_IS_REGULAR)) {
+    GdkColor color;
+    gdk_color_parse("red", &color);
+    gtk_widget_modify_fg(context->fsize, GTK_STATE_NORMAL, &color);
+
     str = g_strdup(_("Not downloaded!"));
     gtk_dialog_set_response_sensitive(GTK_DIALOG(context->dialog), 
 				      GTK_RESPONSE_ACCEPT, 0);
   } else {
+    gtk_widget_modify_fg(context->fsize, GTK_STATE_NORMAL, NULL);
+
     if(!context->project->data_dirty)
       str = g_strdup_printf(_("%d bytes present"), 
 			    file_length(context->project->osm));
@@ -806,37 +851,11 @@ static void on_download_clicked(GtkButton *button, gpointer data) {
 
   printf("download %s\n", context->project->osm);
 
-  gboolean do_it = TRUE;
-  /* check if diff exists and issue a warning */
-  if(diff_present(context->project)) {
-    GtkWidget *dialog = gtk_message_dialog_new(
-	       GTK_WINDOW(context->dialog),
-	       GTK_DIALOG_DESTROY_WITH_PARENT,
-	       GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-	       _("You have saved changes for the current data set. "
-		 "Downloading new data may result in loss of these changes. "
-		 "Do you really want to download new data?"));
-      
-    gtk_window_set_title(GTK_WINDOW(dialog), _("Download?"));
-      
-    /* set the active flag again if the user answered "no" */
-    if(GTK_RESPONSE_NO == gtk_dialog_run(GTK_DIALOG(dialog)))
-      do_it = FALSE;
-    
-    gtk_widget_destroy(dialog);
-  }
-
-  if(!do_it) {
-    printf("Download cancelled\n");
-    return;
-  }
-
   if(osm_download(context->dialog, context->project)) {
     context->project->data_dirty = FALSE;
     project_filesize(context);
   } else
     printf("download failed\n"); 
-
 }
 
 static void on_diff_remove_clicked(GtkButton *button, gpointer data) {
@@ -933,6 +952,7 @@ gboolean project_edit(GtkWidget *parent, project_t *project POS_PARM) {
   gtk_table_attach(GTK_TABLE(table), edit, 3, 4, 2, 4, 
 		   GTK_EXPAND | GTK_FILL,0,0,0);
 
+  gtk_table_set_col_spacing(GTK_TABLE(table), 0, 4);
   gtk_table_set_row_spacing(GTK_TABLE(table), 3, 4);
 
   label = gtk_label_new(_("Server:"));
@@ -997,6 +1017,16 @@ gboolean project_edit(GtkWidget *parent, project_t *project POS_PARM) {
 gboolean project_open(appdata_t *appdata, char *name) {
   project_t *project = g_new0(project_t, 1);
 
+  /* link to map state if a map already exists */
+  if(appdata->map) {
+    printf("Project: Using map state\n");
+    project->map_state = appdata->map->state;
+  } else {
+    printf("Project: Creating new map_state\n");
+    project->map_state = g_new0(map_state_t,1);	      
+  }
+  project->map_state->refcount++;	
+
   /* build project path */
   project->path = g_strdup_printf("%s%s/", 
 		  appdata->settings->base_path, name);
